/*
* Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO USER:
*
* This source code is subject to NVIDIA ownership rights under U.S. and
* international Copyright laws.
*
* NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
* CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
* IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
* REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
* IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
* OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
* OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
* OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
* OR PERFORMANCE OF THIS SOURCE CODE.
*
* U.S. Government End Users.  This source code is a "commercial item" as
* that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of
* "commercial computer software" and "commercial computer software
* documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995)
* and is provided to the U.S. Government only as a commercial end item.
* Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
* 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
* source code with only those rights set forth herein.
*/

#include <nvcuvid.h>
#include <windows.h>

class FrameQueue
{
public:
    static const unsigned int cnMaximumSize = 20; // MAX_FRM_CNT;
    
    FrameQueue();
    
    virtual
   ~FrameQueue();
   
    void
    waitForQueueUpdate();
    
    void
    enqueue(const CUVIDPARSERDISPINFO * pPicParams);
    
    // Deque the next frame.
    // Parameters:
    //      pDisplayInfo - New frame info gets placed into this object.
    //          Note: This pointer must point to a valid struct. The method
    //          does not create memory for this.
    // Returns:
    //      true, if a new frame was returned,
    //      false, if the queue was empty and no new frame could be returned. 
    //          In that case, pPicParams doesn't contain valid data.
    bool
    dequeue(CUVIDPARSERDISPINFO * pDisplayInfo);
    
    void
    releaseFrame(const CUVIDPARSERDISPINFO * pPicParams);
    
    bool
    isInUse(int nPictureIndex)
    const;
    
    bool
    isEndOfDecode()
    const;
    
    void
    endDecode();
    
    // Spins until frame becomes available or decoding
    // gets canceled. 
    // If the requested frame is available the method returns true.
    // If decoding was interupted before the requested frame becomes
    // available, the method returns false.
    bool
    waitUntilFrameAvailable(int nPictureIndex);
    
private:
    void
    signalStatusChange();
    
    HANDLE hEvent_;
    CRITICAL_SECTION    oCriticalSection_;
    volatile int        nReadPosition_;
    volatile int        nFramesInQueue_;
    CUVIDPARSERDISPINFO aDisplayQueue_[cnMaximumSize];
    volatile int        aIsFrameInUse_[cnMaximumSize];
    volatile int        bEndOfDecode_;
};

