/*
* Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO USER:
*
* This source code is subject to NVIDIA ownership rights under U.S. and
* international Copyright laws.
*
* NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
* CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
* IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
* REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
* IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
* OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
* OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
* OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
* OR PERFORMANCE OF THIS SOURCE CODE.
*
* U.S. Government End Users.  This source code is a "commercial item" as
* that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of
* "commercial computer software" and "commercial computer software
* documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995)
* and is provided to the U.S. Government only as a commercial end item.
* Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
* 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
* source code with only those rights set forth herein.
*/

#ifndef NV_IMAGE_DX
#define NV_IMAGE_DX

#include <cuda.h>

#ifdef _WIN32
#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <d3dx9.h>

class ImageDX
{
public:
    enum PixelFormat
    {
        LUMINANCE_PIXEL_FORMAT,
        BGRA_PIXEL_FORMAT,
        UNKNOWN_PIXEL_FORMAT
    };
    
    ImageDX(IDirect3DDevice9 * pDeviceD3D, unsigned int nWidth, unsigned int nHeight, PixelFormat ePixelFormat = BGRA_PIXEL_FORMAT);
    
    // Destructor
   ~ImageDX();
   
    void
    registerAsCudaResource();

    void
    unregisterAsCudaResource();
    
    bool
    isCudaResource()
    const;
    
    // Map this image's DX surface into CUDA memory space.
    // Parameters:
    //      ppImageData - point to point to image data. On return this
    //          pointer references the mapped data.
    //      pImagePitch - pointer to image pitch. On return of this
    //          pointer contains the pitch of the mapped image surface.
    // Note:
    //      This method will fail, if this image is not a registered CUDA resource.
    void
    map(CUdeviceptr * ppImageData, unsigned int * pImagePitch);
    
    void
    unmap();
    
    // Clear the image.
    // Parameters:
    //      nClearColor - the luminance value to clear the image to. Default is white.
    // Note:
    //      This method will not work if this image is not registered as a CUDA resource at the 
    //      time of this call.
    void
    clear(unsigned char nClearColor = 0xff);
    
    unsigned int
    width()
    const;
    
    unsigned int
    height()
    const;
    
    void
    render()
    const;
    
private:
	static
	D3DFORMAT
	d3dFormat(PixelFormat ePixelFormat);
	

	struct VertexStruct
	{
	    VertexStruct() {};
	    VertexStruct(float nX, float nY, float nZ, float nU, float nV);
		float position[3];
		float texture[2];
	};
	
	
	VertexStruct aVertexBuffer_[4];

	static const unsigned int aIndexBuffer_[6];

    IDirect3DDevice9  * pDeviceD3D_;
    IDirect3DTexture9 * pTexture_;
	IDirect3DSurface9 * pSurface_;
	
	unsigned int nWidth_;
	unsigned int nHeight_;	
	
	bool bIsCudaResource_;
};

#endif // NV_IMAGE_DX