/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.  Users and possessors of this source code
 * are hereby granted a nonexclusive, royalty-free license to use this code
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

 /* CUmoduleManager manages loading CUBIN, initializing CUBIN kernel function,
  * initializating CUDA kernel function pointers, and obtaining global memory 
  * addresses (i.e. constants).
  */

#ifndef _CUDAMODULEMGR_H_
#define _CUDAMODULEMGR_H_

#include <memory>
#include <iostream>
#include <cassert>

class CUmoduleManager
{
public:
    // Each time one of this Driver object is instantiated, it loads the CUBIN into a CUmodule
    CUmoduleManager(const char *filename_cubin, const char *exec_path, int nKernels, int nGlobalMem);
    ~CUmoduleManager();

    CUresult GetCudaFunction (const char *func_name, CUfunction *fpCudaKernel = 0);
    CUresult GetCudaDevicePtr(const char *address_name, CUdeviceptr *pGlobalMem = 0);

    int findIndex_CudaKernels(const char *func_name);
    int findIndex_GlobalMem(const char *address_name);

    CUresult launchKernel(CUfunction fpFunc, dim3 block, dim3 grid );

private:
    int nMaxKernels_;  // these two keep track of the maximum number of kernels and constants
    int nMaxGlobalMem_;

    int nLastKernel_;   // these two keep track of the last kernel or constant in the array
    int nLastGlobalMem_;

    struct CudaKernels
    {
        CUfunction fpCuda;
        std::string func_name;
    } * pCudaKernels_;

    struct CudaGlobalMem
    {
        CUdeviceptr devicePtr;
        unsigned int nBytes;
        std::string address_name;
    } * pGlobalMem_;

public:
    CUmodule    cuModule_;
};




#endif
