/*
* Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO USER:
*
* This source code is subject to NVIDIA ownership rights under U.S. and
* international Copyright laws.
*
* NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
* CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
* IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
* REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
* IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
* OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
* OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
* OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
* OR PERFORMANCE OF THIS SOURCE CODE.
*
* U.S. Government End Users.  This source code is a "commercial item" as
* that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of
* "commercial computer software" and "commercial computer software
* documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995)
* and is provided to the U.S. Government only as a commercial end item.
* Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
* 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
* source code with only those rights set forth herein.
*/

#ifndef _CUDAPROCESSFRAME_H_
#define _CUDAPROCESSFRAME_H_

#include <cuda.h>
#include <vector_types.h>

typedef unsigned char   uint8;
typedef unsigned int    uint32;
typedef int             int32;

#ifndef __DRIVER_TYPES_H__
typedef enum cudaError cudaError_t;
#endif

#define COLOR_COMPONENT_MASK            0x3FF
#define COLOR_COMPONENT_BIT_SIZE        10

#define FIXED_DECIMAL_POINT			    24
#define FIXED_POINT_MULTIPLIER		    1.0f
#define FIXED_COLOR_COMPONENT_MASK	    0xffffffff

typedef enum {
    ITU601 = 1,
    ITU709 = 2
} eColorSpace;


// The NV12ToARGB helper functions and CUDA kernel launchers that get called
extern "C" 
{
    CUresult InitCudaModule  (char *filename_cubin, char *exec_path,   CUmodule   *cuModule);
    CUresult InitCudaFunction(char *func_name,      CUmodule *pModule, CUfunction *pCudaFunction);

    CUresult updateConstantMemory        ( float *hueCSC );
    CUresult updateConstantMemory_drvapi ( CUmodule    cuModule, float *hueColorSpaceMat            );
    void     setColorSpaceMatrix         ( eColorSpace      CSC, float *hueColorSpaceMat, float hue );

    CUresult cudaLaunchNV12toARGB    (  uint32 *d_srcNV12, uint32 nSourcePitch,
                                        uint32 *d_dstARGB, uint32 nDestPitch,
                                        uint32 width, uint32 height                );
    CUresult cudaLaunchNV12toARGB_4pix( uint32 *d_srcNV12, uint32 nSourcePitch,
                                        uint32 *d_dstARGB, uint32 nDestPitch,
                                        uint32 width, uint32 height                );

    CUresult cudaLaunchNV12toARGBDrv (  CUdeviceptr d_srcNV12, uint32 nSourcePitch,
                                        CUdeviceptr d_dstARGB, uint32 nDestPitch,
                                        uint32 width, uint32 height,
                                        CUfunction fpNV12toARGB                    );
};


#endif
